using System;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Threading;

namespace Common
{
	/// <summary>
	///     Logging class to provide tracing and logging support. 
	///     <remarks>
	///         There are four different logging levels (error, warning, info, trace) 
	///         that produce output to either the system event log or a tracing 
	///         file as specified in the current ApplicationConfiguration settings.
	///     </remarks>
	/// </summary>
	public class ApplicationLog
	{
		private const long MAX_LOG_FILE_LENGTH = 256 * 1024;

		//
		// Shared member variables
		//

		//     TraceSwitch for changing which values are actually written to the
		//     trace output file.
		private static TraceSwitch debugSwitch;
		//This object is added as a debug listener.
		private static StreamWriter debugWriter;

		//EventLog variables used for the trace level if the event log is specified.
		private static TraceLevel eventLogTraceLevel;

		/// <summary>
		///     Write at the Error level to the event log and/or tracing file.
		///     <param name="message">The text to write to the log file or event log.</param>
		/// </summary>
		public static void WriteError(String message, string fileName)
		{
			//Defer to the helper function to log the message.
			WriteLog(TraceLevel.Error, message, fileName);
		}

		/// <summary>
		///     Write at the Warning level to the event log and/or tracing file.
		///     <param name="message">The text to write to the log file or event log.</param>
		/// </summary>
		public static void WriteWarning(String message, string fileName)
		{
			//Defer to the helper function to log the message.
			WriteLog(TraceLevel.Warning, message, fileName);
		}

		/// <summary>
		///     Write at the Info level to the event log and/or tracing file.
		///     <param name="message">The text to write to the log file or event log.</param>
		/// </summary>
		public static void WriteInfo(String message, string fileName)
		{
			//Defer to the helper function to log the message.
			WriteLog(TraceLevel.Info, message, fileName);
		}

		/// <summary>
		///     Write at the Verbose level to the event log and/or tracing file.
		///     <param name="message">The text to write to the log file or event log.</param>
		/// </summary>
		public static void WriteTrace(String message, string fileName)
		{
			//Defer to the helper function to log the message.
			WriteLog(TraceLevel.Verbose, message, fileName);
		}

		/// <summary>
		///     Write at the Verbose level to the event log and/or tracing file.
		///     <param name="ex">The Exception object to format</param>
		///     <param name="catchInfo">The string to prepend to the exception information.</param>
		///     <retvalue>
		///         <para>A nicely format exception string, including message and StackTrace information.</para>
		///     </retvalue>
		/// </summary>
		public static String FormatException(Exception ex, String catchInfo)
		{
			StringBuilder strBuilder = new StringBuilder();
			if (catchInfo != String.Empty)
			{
				strBuilder.Append(catchInfo).Append("\r\n");
			}

			if (System.Web.HttpContext.Current != null && System.Web.HttpContext.Current.Request != null)
			{
				strBuilder.Append("URL:            ");
				strBuilder.Append(System.Web.HttpContext.Current.Request.RawUrl);
				strBuilder.Append("\r\nREMOTE HOST:    ");
				strBuilder.Append(System.Web.HttpContext.Current.Request.ServerVariables["REMOTE_ADDR"]);
				strBuilder.Append(":");
				strBuilder.Append(System.Web.HttpContext.Current.Request.ServerVariables["REMOTE_PORT"]);
				strBuilder.Append("\r\nCOOKIE:         ");
				strBuilder.Append(System.Web.HttpContext.Current.Request.Headers["Cookie"]);
				strBuilder.Append("\r\n");
			}

			strBuilder.Append("An exception of type ");
			strBuilder.Append(ex.GetType().Name);
			strBuilder.Append(" was thrown ");
			strBuilder.Append("\r\n");
			strBuilder.Append("Application name: ");
			strBuilder.Append(ex.Source);
			strBuilder.Append("\r\n");
			strBuilder.Append(ex.Message);
			strBuilder.Append("\r\n");
			Exception tmpExc = GetMostInnerException(ex);
			if (tmpExc != ex)
			{
				strBuilder.Append("Inner exception of type ");
				strBuilder.Append(tmpExc.GetType().Name);
				strBuilder.Append(" was thrown ");
				strBuilder.Append("\r\n");
				strBuilder.Append("Application name: ");
				strBuilder.Append(tmpExc.Source);
				strBuilder.Append("\r\n");
				strBuilder.Append(tmpExc.Message);
				strBuilder.Append("\r\n");
				strBuilder.Append(tmpExc.StackTrace);
				strBuilder.Append("\r\n");
			}
			strBuilder.Append(ex.StackTrace);
			strBuilder.Append("\r\n");
			strBuilder.Append("\r\n");
			strBuilder.Append("-----------------------------------------------------------------------------------------------------------------------");
			return strBuilder.ToString();
		}

		public static Exception GetMostInnerException(Exception exc)
		{
			Exception inner = exc;
			for (int i = 0; i < 100; i++)
			{
				if (inner.InnerException == null)
					break;
				else
					inner = inner.InnerException;
			}

			return inner;
		}

		/// <summary>
		///     Determine where a string needs to be written based on the
		///     configuration settings and the error level.
		///     <param name="level">The severity of the information to be logged.</param>
		///     <param name="messageText">The string to be logged.</param>
		/// </summary>
		private static void WriteLog(TraceLevel level, String messageText, string fileName)
		{
			System.IO.StreamWriter logWriter = null;
			System.IO.FileStream logFile = null;

			try
			{
				//				if (ApplicationConfiguration.TracingEnabled)
				//				{
				logFile = new FileStream(fileName, System.IO.FileMode.Append, System.IO.FileAccess.Write, System.IO.FileShare.ReadWrite);
				logWriter = new StreamWriter(logFile);
				logWriter.Write("\r\n" + DateTime.Now.ToString() + "\r\n" + messageText);
				logWriter.Close();
				logFile.Close();
				//				}

			}
			catch (Exception ex)
			{
				string a = ex.Message;
				if (logWriter != null)
					logWriter.Close();
				if (logFile != null)
					logFile.Close();
			}
		}

		/// <summary>
		///     Constructor for ApplicationLog.  
		///     <remarks>Initialize all shared variables based on the ApplicationConfigurationsettings. 
		///         Called when this class is first loaded.
		///     </remarks>
		/// </summary>
		static ApplicationLog()
		{
			//
			// Read the current settings from the configuration file to determine
			//   whether we need trace file support, event logging, or both.
			//

			//Get the class object in order to take the initialization lock
			Type myType = typeof(ApplicationLog);

			//Protect thread locks with Try/Catch to guarantee that we let go of the lock.
			try
			{
				//See if anyone else is using the lock, grab it if they//re not
				if (!Monitor.TryEnter(myType))
				{
					//Just wait until the other thread finishes processing, then leave if
					//  the lock was already in use.
					Monitor.Enter(myType);
					return;
				}

				//See if there is a debug configuration file specified and set up the
				//  tracing variables.
				bool clearSettings = true;
				try
				{
					//Check if we're enabled

					if (ApplicationConfiguration.TracingEnabled)
					{
						//Make sure we have a TraceSettings file.
						String tracingFile = ApplicationConfiguration.TracingTraceFile;
						if (tracingFile != String.Empty)
						{

							//Read in the tracing switch name and create the switch.
							String switchName = ApplicationConfiguration.TracingSwitchName;

							//Create the new switch
							if (switchName != String.Empty)
							{
								//Create a debug listener and add it as a debug listener
								FileInfo file = new FileInfo(tracingFile);
								debugWriter = new StreamWriter(file.Open(FileMode.Append, FileAccess.Write, FileShare.ReadWrite));
								Debug.Listeners.Add(new TextWriterTraceListener(debugWriter));

								debugSwitch = new TraceSwitch(switchName, ApplicationConfiguration.TracingSwitchDescription);
								debugSwitch.Level = ApplicationConfiguration.TracingTraceLevel;
							}
							clearSettings = false;
						}
					}
				}
				catch
				{
					//Ignore the error
				}

				//Use default (empty) values if something went wrong
				if (clearSettings)
				{
					//Tracing information is off or not properly specified, clear it
					debugSwitch = null;
					debugWriter = null;
				}

				if (ApplicationConfiguration.EventLogEnabled)
					eventLogTraceLevel = ApplicationConfiguration.EventLogTraceLevel;
				else
					eventLogTraceLevel = TraceLevel.Off;
			}
			finally
			{
				//Remove the lock from the class object
				Monitor.Exit(myType);
			}
		}

		public static void CheckLogFile(string fileName)
		{
			if (!File.Exists(fileName)) return;

			FileInfo fileInfo = new System.IO.FileInfo(fileName);
			if (fileInfo.Length < MAX_LOG_FILE_LENGTH) return;

			string backupFileName = Path.ChangeExtension(fileName, "bak");
			if (File.Exists(backupFileName)) File.Delete(backupFileName);
			fileInfo.MoveTo(backupFileName);
		}
		public static void LogException(Exception exc, string info)
		{
			WriteLog(TraceLevel.Error, FormatException(exc, info), ApplicationConfiguration.TracingTraceFile);
		}

		public static string ReadLogFile()
		{
			return ReadFile(ApplicationConfiguration.TracingTraceFile);
		}
		private static string ReadFile(string fileName)
		{
			System.IO.StreamReader logReader = null;
			System.IO.FileStream logFile = null;

			try
			{
				logFile = new FileStream(fileName, System.IO.FileMode.Open, System.IO.FileAccess.Read, System.IO.FileShare.ReadWrite);
				logReader = new StreamReader(logFile);
				string content = logReader.ReadToEnd();
				logReader.Close();
				logFile.Close();
				return content;
			}
			catch (Exception exc)
			{
				if (logReader != null)
					logReader.Close();
				if (logFile != null)
					logFile.Close();
				return exc.Message;
			}
		}

	} //class ApplicationLog
} //namespace WVL.SystemFramework
