using System;
using System.Collections.Generic;
using System.Text;

namespace Common
{
	#region Page History
	/// <summary>
	/// This class is used to hold a history of the URLs and their corresponding pages visited by the client.
	/// The main purpose is to allow navigation to previosly visited pages, a limited number of steps back.
	/// This class is serializable because its instance is put in the session state as a part of the User object.
	/// The main data structure is cyclic stack with initial size 10 (allows up to 10 pages back)
	/// </summary>
	[Serializable]
	public class PageHistory
	{
		#region Constants
		/// <summary>
		/// Size of the cyclic stack
		/// </summary>
		public const int SIZE = 10;
		/// <summary>
		/// URL of page to return when stack is empty. Value is Default.aspx
		/// </summary>
		public const string DEFAULT_PAGE = "~/Default.aspx";
		#endregion

		#region Fields
		/// <summary>
		/// An array with the names of the pages visited
		/// </summary>
		private string[] names = new string[SIZE];
		/// <summary>
		/// An array with the URLs of the pages visited
		/// </summary>
		private string[] urls = new string[SIZE];
		/// <summary>
		/// Current size of the stack
		/// </summary>
		private int length = 0;
		/// <summary>
		/// Marks the stack bottom element
		/// </summary>
		private int first = 0;
		#endregion

		#region Properties

		/// <summary>
		/// gets the current stack size
		/// </summary>
		public int Length
		{
			get
			{
				return length;
			}
		}

		/// <summary>
		/// Indexer gets the first url from page name.
		/// </summary>
		public string this[string pageName]
		{
			get
			{
				int idx = Find(pageName);
				if (idx < 0)
					return DEFAULT_PAGE;
				//return null;
				return urls[idx];
			}
		}
		#endregion

		#region Methods
		/// <summary>
		/// gets the name of the page on the top of the stack
		/// </summary>
		/// <returns></returns>
		public string Peek()
		{
			if (length == 0)
				return DEFAULT_PAGE;
			//return null;

			int last = GetPrev(first + length);
			return names[last];
		}

		/// <summary>
		/// Removes the top element of the stack and returns its URL
		/// </summary>
		/// <returns></returns>
		public string Pop()
		{
			string pageName;
			return Pop(out pageName);
		}

		/// <summary>
		/// Returns the URL of the top stack element and the name of the page for this URL
		/// </summary>
		/// <param name="pageName"></param>
		/// <returns></returns>
		public string Pop(out string pageName)
		{
			if (length == 0)
			{
				pageName = "";
				return DEFAULT_PAGE;
				//return null;
			}

			int last = GetPrev(first + length);
			string result = urls[last];
			pageName = names[last];
			urls[last] = null;
			names[last] = null;
			length--;
			if (result == null)
				return DEFAULT_PAGE;
			return result;
		}

		/// <summary>
		/// Adds a new item in the history
		/// </summary>
		/// <param name="pageName"></param>
		/// <param name="url"></param>
		public void Push(string pageName, string url)
		{
			if (pageName == null)
				return;

			pageName = pageName.ToUpper();
			if (length > 0 && Peek() == pageName)
			{
				int last = GetPrev(first + length);
				names[last] = pageName;
				urls[last] = url;
			}
			else
			{
				if (length == SIZE)
				{
					names[first] = pageName;
					urls[first] = url;
					first = GetNext(first);
				}
				else
				{
					int last = GetPrev(first + length);
					last = GetNext(last);
					names[last] = pageName;
					urls[last] = url;
					length++;
				}
			}
		}

		/// <summary>
		/// Searches for the first item in the history that has the specified page name and returns its ID if found
		/// </summary>
		/// <param name="pageName"></param>
		/// <returns></returns>
		public int Find(string pageName)
		{
			if (pageName == null)
				return -1;

			pageName = pageName.ToUpper();
			for (int i = 0; i < SIZE; i++)
			{
				if (names[i] != null && names[i] == pageName)
					return i;
			}
			return -1;
		}

		/// <summary>
		/// Gets the preceding page of the one that has the name specified
		/// </summary>
		/// <param name="pageName"></param>
		/// <returns></returns>
		public string GetPrevPage(string pageName)
		{
			if (length == 0)
				//return null;
				return DEFAULT_PAGE;

			pageName = pageName.ToUpper();
			string page;
			for (; ; )
			{
				if (length == 1)
					return Pop();
				Pop(out page);
				if (page == pageName)
					return Pop();
			}
		}

		/// <summary>
		/// Gets the ID of the next element in the cyclic stack
		/// </summary>
		/// <param name="n"></param>
		/// <returns></returns>
		private int GetNext(int n)
		{
			return (n + 1) % SIZE;
		}
		/// <summary>
		/// Gets the ID of the previous element in the cyclic stack
		/// </summary>
		/// <param name="n"></param>
		/// <returns></returns>
		private int GetPrev(int n)
		{
			return (n + SIZE - 1) % SIZE;
		}
		#endregion
	}
	#endregion
}
