using System;
using System.Drawing;

namespace Common
{
	public class ParseMethods
	{
		#region BadValue Consts
		public static readonly int VALUE_BAD_INT = int.MinValue;
		public static readonly float VALUE_BAD_FLOAT = float.MinValue;
		public static readonly double VALUE_BAD_DOUBLE = double.MinValue;
		public static readonly DateTime VALUE_BAD_DATE = DateTime.MinValue;
		public static readonly bool VALUE_BAD_BOOL = false;
		public static readonly TimeSpan VALUE_BAD_TIME = TimeSpan.MinValue;
		public static readonly long VALUE_BAD_LONG = long.MinValue;
        public static readonly Color VALUE_BAD_COLOR = Color.Empty;
		#endregion

		public static object ConvertToType(object val, Type type)
		{
			if (type == typeof(int))
				return ParseMethods.ReadIntZero(val);
			if (type == typeof(string))
				return val == null ? null : val.ToString();
			if (type == typeof(long))
				return ParseMethods.ReadLong(val);
			if (type == typeof(DateTime))
				return ReadDate(val);
			if (type == typeof(float))
				return ReadFloatZero(val);
			if (type == typeof(bool))
				return ReadBool(val);
			return val;
		}

        public static Color ReadColor(string strHex)
        {
            if (strHex == null || strHex == "")
                return VALUE_BAD_COLOR;
            try
            {
                if (!strHex.StartsWith("#"))
                    strHex = "#" + strHex;
                return ColorTranslator.FromHtml(strHex);
            }
            catch
            {
                return VALUE_BAD_COLOR;
            }

        }

		public static long ReadLong(object val)
		{
			if (val == null || val == System.DBNull.Value) return VALUE_BAD_LONG;
			if (val is long) return (long)val;

			string input = val.ToString().Trim();
			if (input == "") return VALUE_BAD_LONG;
			try
			{
				return long.Parse(input, System.Globalization.NumberStyles.Any);
			}
			catch
			{
				return VALUE_BAD_LONG;
			}
		}

        public static long ReadLongZero(object val)
        {
            long tmp = ReadLong(val);
            if (tmp == VALUE_BAD_LONG)
                return 0;
            return tmp;
        }

		public static int ReadInt(object val)
		{
			if (val == null || val == System.DBNull.Value) return VALUE_BAD_INT;
			if (val is int) return (int)val;

			string input = val.ToString().Trim();
			if (input == "") return VALUE_BAD_INT;
			try
			{
				return Int32.Parse(input, System.Globalization.NumberStyles.Any);
			}
			catch
			{
				return VALUE_BAD_INT;
			}
		}

		public static int ReadIntZero(object val)
		{
			if (val == null || val == System.DBNull.Value) return 0;
			if (val is int) return (int)val;

			string input = val.ToString().Trim();
			if (input == "") return 0;
			try
			{
				return Int32.Parse(input, System.Globalization.NumberStyles.Any);
			}
			catch
			{
				return 0;
			}
		}


		public static float ReadFloat(object val)
		{
			if (val == null || val == System.DBNull.Value) return VALUE_BAD_FLOAT;
			if (val is float) return (float)val;

			string input = val.ToString().Trim();
			if (input == "") return VALUE_BAD_FLOAT;
			try
			{
				return float.Parse(input, System.Globalization.NumberStyles.Any);
			}
			catch
			{
				return VALUE_BAD_FLOAT;
			}
		}

		public static double ReadDouble(object val)
		{
			if (val == null || val == System.DBNull.Value) return VALUE_BAD_DOUBLE;
			if (val is double) return (double)val;

			string input = val.ToString().Trim();
			if (input == "") return VALUE_BAD_DOUBLE;
			try
			{
				return double.Parse(input, System.Globalization.NumberStyles.Any);
			}
			catch
			{
				return VALUE_BAD_DOUBLE;
			}
		}

		public static float ReadFloatZero(object val)
		{
			if (val == null || val == System.DBNull.Value) return 0;
			if (val is float) return (float)val;

			string input = val.ToString().Trim().Replace(",", System.Globalization.NumberFormatInfo.CurrentInfo.NumberDecimalSeparator).Replace(".", System.Globalization.NumberFormatInfo.CurrentInfo.NumberDecimalSeparator);
			if (input == "") return 0;
			try
			{
				return float.Parse(input, System.Globalization.NumberStyles.AllowDecimalPoint);
			}
			catch
			{
				return 0;
			}
		}

        public static DateTime ReadDate(object input)
        {
            return ReadDateBg(input);
        }

		public static DateTime ReadDateBg(object input)
		{
			if (input == null || input.ToString() == "")
				return VALUE_BAD_DATE;

            if (input is DateTime)
                return (DateTime)input;

			DateTime result;
			try
			{
                result = DateTime.ParseExact(input.ToString(), "dd/MM/yyyy", null);
			}
			catch 
			{
                return VALUE_BAD_DATE;
			}

			return result;
		}

		public static bool ReadBool(object param)
		{
			if (param == null || param == System.DBNull.Value)
				return VALUE_BAD_BOOL;
			try
			{
				string input = param.ToString().Trim();
				switch (input.ToUpper())
				{
					case "1":
					case "T":
					case "Y":
					case "J":
						return true;
					case "0":
					case "F":
					case "N":
						return false;
				}

				return bool.Parse(input);
			}
			catch
			{
				return VALUE_BAD_BOOL;
			}
		}

		public static int ReadBoolAsInt(object input)
		{
			if (input != null && ReadBool(input))
				return 1;
			return 0;
		}

		public static DateTime ReadDateTimeBg(object input)
		{
            if (input == null || input.ToString() == "")
                return VALUE_BAD_DATE;

            if (input is DateTime)
                return (DateTime)input;

            DateTime result;
            try
            {
                result = result = DateTime.ParseExact(input.ToString(), "dd/MM/yyyy hh:mm", null);
            }
			catch
			{
                return VALUE_BAD_DATE;
			}
            return result;
		}

        public static int ParseEnum(Type enumType, object val)
        {
            int valDefault = 0;
            Array arr = Enum.GetValues(enumType);
            if (arr.Length > 0)
                valDefault = (int)arr.GetValue(0);
            if (!Enum.IsDefined(enumType, val))
                return valDefault;

            try
            {
                return (int)Enum.Parse(enumType, val.ToString(), true);
            }
            catch
            {
                return valDefault;
            }
        }
	}
}
