using System;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;

namespace Common
{

	#region Trace
	/// <summary>
	/// This class is used to provide tracing functionality for the Opptak system.
	/// This trace log is different from the application error log.
	/// Its methods are called from the main web application event handlers in the global.asax.
	///	It is configurable in the web.config whether the application should trace its activity.
	///	By defaul the trace log is saved to the trace folder of the solution where the ASP process need to have 
	///	enough rights to create and modify the contents of a file.
	///	These result could be imported in Excel and customized by the end user,
	///	however the main puropose of here is to gather data with the least possible overhead.
	///	Here are the lines from the config file:
	///	<code>
	///	<add key="TraceFile" value="\trace\trace.txt" />	
	///	<add key="TraceEnabled" value="true" />	
	///	</code>		
	///	Below there is a sample record:
	/// </summary>
	/// <example>
	/// Field						Sample value				Explanation
	/// -------------------------------------------------------------------
	/// ApplicationStartTime		25.05.2005 11:55:27			When was the application started - equals the time of the first request to the application
	/// Url							/Login.aspx	    	    	The name of the page that was visited
	/// InstanceClassifier			Login_aspx		    		Name of the class from the UI layer that produced the output to the request. This one may differ from the one from the URL if there was a redirect. 
	///															In case we want the class name of the request handler we can extract it from the Context
	/// RequestDuration				0,0468774					How many seconds were needed to produce the ouput
	/// RequestDate					26.05.2005					Date ofthe current request
	/// RequestBeginTime			11:55:50					At what time there was made the request for the web form
	/// SessionStartTime			--------------				If the current session was started with the current request, here is the time the session is strated
	/// SessionID					03jw543plgw1fnevbbtlid45	ID of session
	/// SessionCount				2							How many sessions are currently active. In case of out of process state management this count will only increase because the session-end event is not fired in that case
	/// RequestID					1314						ID of the current request (unique for this application)
	/// RequestSize					N/A							Not implemented. No criteria for measurement
	/// ResponseSize				N/A							Not implemented. No criteria for measurement
	/// SimultaneousRequests		1							How many requests does the system handle together with the current request. The current is taken into account
	/// SimRequest/Session			1							How many requests does the system handle for that session.
	/// QueryStringText				param=4E373D9252C82D...		Parameters of the query string
	/// ErrorMsg												In case of exception here is the msg
	///                                                                                                                                   
	/// </example>
	public class Trace
	{
		#region LogData Class
		
		public class LogData
		{
			#region fields
			private DateTime requestBeginTime;
			private DateTime requestEndTime;
			private DateTime sessionStartTime;
			private DateTime applicationStartTime;
			private long requestSize;
			private long responseSize;						
			private string url;
			private string instanceClassifier;
			private string rawUrl;
			private string sessionID;
			private string errorMsg;
			/// <summary>
			/// Primary Key of the Log data - to be found in the LogData Collection of the Trace class. See <see cref="Trace"/>
			/// </summary>
			public readonly int RequestID;
			private Hashtable logDataCollection;
			private Hashtable sessionCountCollection;
			private static int simultaneousRequests;			
			#endregion

			#region public properties
			/// <summary>
			/// Gets/sets number of simultaneous requests
			/// </summary>
			public static int SimultaneousRequests
			{
				get
				{
					return simultaneousRequests;
				}
				set
				{
					simultaneousRequests = value; 
				}
			}

			/// <summary>
			/// Gets the request date
			/// </summary>
			public string RequestDate
			{
				get
				{
					return requestBeginTime.ToShortDateString();
				}
			}

			/// <summary>
			/// Gets the time when the request began
			/// </summary>
			public string RequestBeginTime
			{
				get
				{
					return requestBeginTime.ToLongTimeString();
				}			
			}

			/// <summary>
			/// Gets the time the request ended
			/// </summary>
			public string RequestEndTime
			{
				get
				{
					return requestEndTime.ToLongTimeString();
				}
			}

			/// <summary>
			/// Gets the request duration
			/// </summary>
			public long RequestDuration
			{
				get
				{
					return (requestEndTime-requestBeginTime).Ticks;
				}				
			}

			/// <summary>
			/// Gets the start time of the current session
			/// </summary>
			public string SessionStartTime
			{
				get
				{
					if (sessionStartTime == DateTime.MinValue)
						return "--------------";
					return sessionStartTime.ToLongTimeString();
				}				
			}

			/// <summary>
			/// Gets the strat date of the application
			/// </summary>
			public string ApplicationStartDate
			{
				get
				{
					return applicationStartTime.ToShortDateString();
				}				
			}

			/// <summary>
			/// Gets the start time of the aplication
			/// </summary>
			public string ApplicationStartTime
			{
				get
				{
					return applicationStartTime.ToShortDateString() + " " + applicationStartTime.ToLongTimeString();
				}			
			}

			/// <summary>
			/// Gets/Sets the size of the request. Not yet implemented
			/// </summary>
			public long RequestSize
			{
				get
				{
					return requestSize;
				}
				set
				{
					requestSize = value;
				}
			}

			/// <summary>
			/// Gets/sets the size of the response. Not yet implemented
			/// </summary>
			public long ResponseSize
			{
				get
				{
					return responseSize;
				}
				set
				{
					responseSize = value;
				}
			}
			
			/// <summary>
			/// Gets/sets the URL for the current request
			/// </summary>
			public string Url
			{
				get
				{
					int idx2 = url.IndexOf('?');
					if (idx2<0)
						idx2=url.Length;

					string tmp;
                    
					tmp = url.Substring(0,idx2);
					int idx = tmp.LastIndexOf('/');		
					int length = idx2-idx;
					length = length>0?length:url.Length-idx;
					
					tmp  = url.Substring(idx,length);
					
					return tmp;
				}
				set
				{
					url = value;
				}
			}

			/// <summary>
			/// Gets/sets the instance classifier of the page that produced the output to the client
			/// </summary>
			public string InstanceClassifier
			{
				get
				{
					return instanceClassifier;
				}
				set
				{
					instanceClassifier = value;
				}
			}

			/// <summary>
			/// Gets/sets the raw URL of the request
			/// </summary>
			public string RawUrl
			{
				get
				{
					return rawUrl;
				}
				set
				{
					rawUrl = value;
				}
			}

			/// <summary>
			/// Gets the error message in case any problem occurs
			/// </summary>
			public string ErrorMsg
			{
				get
				{
					return errorMsg;
				}
				set
				{
					errorMsg = value;
				}
			}

			/// <summary>
			/// Gets the number of sessions. This count is correct only in InProc mode
			/// </summary>
			public int SessionCount
			{
				get
				{
					return sessionCountCollection.Count;
				}
			}

			/// <summary>
			/// Gets the parameter part of the query string
			/// </summary>
			public string QueryStringText
			{
				get
				{
					int idx = url.LastIndexOf('?');
					if (idx<0)
						return "";
					return url.Substring(idx+1,url.Length-idx-1);
				}
			}

			/// <summary>
			/// Gets/sets the session ID for the request
			/// </summary>
			public string SessionID
			{
				get
				{
					return sessionID;
				}
				set
				{
					sessionID = value;
				}

			}

			/// <summary>
			/// Gets the number of simultanious request performed from the current session. This count is correct only in InProc mode
			/// </summary>
			public int SimultaneousRequestsFromSession
			{
				get
				{
					if (sessionID == "" || sessionCountCollection[sessionID]==null)
						return 0;

					return (int)sessionCountCollection[sessionID];
				}				
			}			
			#endregion

			#region Constructors/Destructors/Initializers
			/// <summary>
			/// Default constructor
			/// </summary>
			public LogData()
			{}

			/// <summary>
			/// Initializes the inner properties that correspond to the parameter values passed to the constructor
			/// </summary>
			/// <param name="appStartTime"></param>
			/// <param name="reqID"></param>
			/// <param name="logDataCollection"></param>
			/// <param name="sessionCountCollection"></param>
			public LogData(DateTime appStartTime, int reqID, Hashtable 
				logDataCollection, Hashtable sessionCountCollection)
			{
				this.applicationStartTime = appStartTime;
				this.RequestID = reqID;
				this.sessionCountCollection = sessionCountCollection;
				this.logDataCollection = logDataCollection;
			}
			#endregion

			#region Set

			/// <summary>
			/// Sets the time when the request began			
			/// </summary>
			/// <param name="time"></param>
			public void SetRequestBeginTime(DateTime time)
			{
				requestBeginTime = time;
			}

			/// <summary>
			/// Sets the request begin time property
			/// </summary>
			/// <param name="time"></param>
			public void SetRequestEndTime(DateTime time)
			{
				requestEndTime = time;
			}

			/// <summary>
			/// Sets the session start time property
			/// </summary>
			/// <param name="time"></param>
			public void SetSessionStartTime(DateTime time)
			{
				sessionStartTime = time;
			}

			/// <summary>
			/// Adds a session ID to the session count tracking collection
			/// </summary>
			#endregion

			#region Manipulation
			public void AddSession()
			{
				if (this.sessionID==null || this.sessionID=="")
					return;

				if (sessionCountCollection[this.sessionID]==null)
				{
					sessionCountCollection.Add(this.sessionID,1);
					return;
				}
				sessionCountCollection[this.sessionID] = (int)sessionCountCollection[this.sessionID]+1;
			}
			/// <summary>
			/// Removes a session ID from the session count tracking collection
			/// </summary>
			public void RemoveSession()
			{
				if (this.sessionID==null || this.sessionID=="")
					return;
				if (sessionCountCollection[this.sessionID]==null)
				{					
					return;
				}
				sessionCountCollection[this.sessionID] = (int)sessionCountCollection[this.sessionID]-1;
			}

			/// <summary>
			/// Gets the column names when creating a new file
			/// </summary>
			/// <returns></returns>
			#endregion

			#region Output file
			public static string GetFileHeader()
			{
				System.Text.StringBuilder builder = new System.Text.StringBuilder();
				builder.Append(StringFormat("ApplicationStartTime",30));
				builder.Append(StringFormat("Url",50));
				builder.Append(StringFormat("InstanceClassifier",30));
				builder.Append(StringFormat("RequestDuration",30));
				builder.Append(StringFormat("RequestDate",30));
				builder.Append(StringFormat("RequestBeginTime",30));								
				builder.Append(StringFormat("SessionStartTime",30));				
				builder.Append(StringFormat("SessionID",30));
				builder.Append(StringFormat("SessionCount",30));
				//builder.Append(this.RawUrl);
				//builder.Append('\t');				
				builder.Append(StringFormat("RequestID",30));				
				builder.Append(StringFormat("RequestSize",30));
				builder.Append(StringFormat("ResponseSize",30));					
				builder.Append(StringFormat("SimultaneousRequests",30));
				builder.Append(StringFormat("SimRequest/Session",30));
				builder.Append(StringFormat("QueryStringText",100));
				builder.Append(StringFormat("ErrorMsg",100));
				//				builder.Append("User.ActiveRoleID");
				//				builder.Append('\t');
				//				builder.Append("User.UserID");
				//				builder.Append('\t');						
                
				return builder.ToString();
			}

			/// <summary>
			/// Reaturns a string representation of the log data to be written in the trace file
			/// </summary>
			/// <returns></returns>
			public override string ToString()
			{			
				System.Text.StringBuilder builder = new System.Text.StringBuilder();
				builder.Append(StringFormat(this.ApplicationStartTime,30));	
				builder.Append(StringFormat(this.Url,50));	
				builder.Append(StringFormat(this.InstanceClassifier,30));
				builder.Append(StringFormat((this.RequestDuration/10000000f).ToString(),30));
				builder.Append(StringFormat(this.RequestDate,30));
				builder.Append(StringFormat(this.RequestBeginTime,30));								
				builder.Append(StringFormat(this.SessionStartTime,30));						
				builder.Append(StringFormat(this.SessionID,30));
				builder.Append(StringFormat(this.SessionCount.ToString(),30));
				builder.Append(StringFormat(this.RequestID.ToString(),30));
				builder.Append(StringFormat(this.RequestSize.ToString(),30));
				builder.Append(StringFormat(this.ResponseSize.ToString(),30));
				builder.Append(StringFormat(SimultaneousRequests.ToString(),30));
				builder.Append(StringFormat(this.SimultaneousRequestsFromSession.ToString(),30));
				builder.Append(StringFormat(this.QueryStringText,100));
				builder.Append(StringFormat(this.ErrorMsg,100));									
				
				return  builder.ToString();
			}			
			#endregion
		}

		/// <summary>
		/// Formats a string to fix string length
		/// </summary>
		/// <param name="input"></param>
		/// <param name="minLength"></param>
		/// <returns></returns>
		public static string StringFormat(object input, int minLength)
		{
			return String.Format("{0,-"+minLength+":S}",input);			
		}

		#endregion

		#region Properties
		/// <summary>
		/// Retrieves/sets the log file name
		/// </summary>
		public static string LogFileName
		{
			get
			{
				if (logFileName=="")
                    return System.Web.HttpContext.Current.Server.MapPath("~") + ApplicationConfiguration.AppSettings[StringConstants.CONFIG_KEY_TRACE_FILE];
				return logFileName;
			}
			set
			{
				logFileName = value;
			}
		}

		/// <summary>
		/// Gets a value indicating whether the tracing is enabled
		/// </summary>
		public static bool TracingEnabled
		{
			get
			{
				return traceEnabled;				
			}
		}

		#endregion

		#region Fields
		private DateTime applicationStartTime;
		private Hashtable logDataCollection = null;
		private Hashtable sessionCountCollection = null;
		private static readonly bool traceEnabled = false;
		private static string logFileName = "";
		#endregion

		#region Help Methods
		#region Application Events
		/// <summary>
		/// Sets the instance classifier property of the log data for this request
		/// </summary>
		/// <param name="page"></param>
		/// <param name="Request"></param>
		public void SetInstanceClassifier(System.Web.UI.Page page,HttpRequest Request)
		{
			LogData logData = logDataCollection[Request.GetHashCode()] as LogData;
			logData.InstanceClassifier = page.GetType().Name;
		}

		/// <summary>
		/// Initializes the trace process. Called on application-start
		/// </summary>
		/// <returns></returns>
		public static Trace ApplicationStart()
		{
			return new Trace();
		}

		/// <summary>
		/// Called on session-start. Sets the session start time and session ID properties of the logger
		/// </summary>
		/// <param name="Session"></param>
		/// <param name="Request"></param>
		public void SessionStart(HttpSessionState Session, HttpRequest Request)
		{			
			LogData logData = logDataCollection[Request.GetHashCode()] as LogData;
			if (logData!=null)
			{
				logData.SessionID = Session.SessionID;
				logData.SetSessionStartTime(DateTime.Now);
			}			
		}

		/// <summary>
		/// Called on application-begin-request. Sets the URL and counts the simultanious requests
		/// </summary>
		/// <param name="Request"></param>
		public void ApplicationBeginRequest(HttpRequest Request)
		{
			LogData logData = new 
				LogData(applicationStartTime,Request.GetHashCode(),logDataCollection, 
				sessionCountCollection);
			logDataCollection.Add(logData.RequestID,logData);

			logData.RawUrl = Request.RawUrl;
			logData.Url = Request.Url.ToString();
			logData.RequestSize = Request.ContentLength;
			LogData.SimultaneousRequests ++;
            
			//logData.ResponseSize = Request.Response.OutputStream.Length;

			logData.SessionID = GetSessionID(Request);
			logData.AddSession();
			logData.SetRequestBeginTime(DateTime.Now);					
		}

		/// <summary>
		/// Called on application-end-request. Prepares the log data object for the request for save and writes a line in the log file.
		/// </summary>
		/// <param name="Request"></param>
		/// <param name="Response"></param>
		public void ApplicationEndRequest(HttpRequest Request, HttpResponse Response)
		{
			int requestHash = Request.GetHashCode();
			LogData logData = logDataCollection[requestHash] as LogData;
			if (logData!=null)
			{
				logData.SetRequestEndTime(DateTime.Now);	
				//logData.Context = GetContext(Request);								
				WriteLog(Trace.LogFileName,logDataCollection[requestHash] as LogData);
				logData.RemoveSession();
				LogData.SimultaneousRequests --;
				logDataCollection.Remove(requestHash);
				
				//sessionCountCollection[logData.SessionID] = (int)sessionCountCollection[logData.SessionID]-1;				
			}
		}

		/// <summary>
		/// No log events associated. Called on authenticate-request.
		/// </summary>
		public void ApplicationAuthenticateRequest()
		{

		}


		/// <summary>
		/// Sets the error message in case of an exception. Called on Application-On-Error
		/// </summary>
		/// <param name="Request"></param>
		/// <param name="exc"></param>
		public void ApplicationError(HttpRequest Request,Exception exc)
		{
			LogData logData = logDataCollection[Request.GetHashCode()] as LogData;
			if (logData!=null)
			{
				logData.ErrorMsg = exc.Message;
			}
		}

		/// <summary>
		/// Called on Session-End. This event is fired only when in InProc mode of the session state.
		/// Helps to decrease the number of not active sessions
		/// </summary>
		/// <param name="Session"></param>
		public void SessionEnd(HttpSessionState Session)
		{
			//HttpSessionState session = GetContext(Request).Session;
			sessionCountCollection.Remove(Session.SessionID);
		}

		/// <summary>
		/// CAlled on application-end
		/// </summary>
		public void ApplicationEnd()
		{
			
		}
		#endregion

		/// <summary>
		/// Retrieves the session ID from the current request.
		/// </summary>
		/// <param name="Request"></param>
		/// <returns></returns>
		public static string GetSessionID(HttpRequest Request)
		{
			if (Request["AspSession"]!=null)
			{
				return Request["AspSession"];
			}
			else if (Request["Asp.NET_SessionId"]!=null)
			{
				return Request["Asp.NET_SessionId"];
			}			

			return null;
		}
		private void RemoveSession(string sesID)
		{
			if (sesID==null || sesID=="")
				return;
			if (sessionCountCollection[sesID]==null)
			{					
				return;
			}
			sessionCountCollection[sesID] = (int)sessionCountCollection[sesID]-1;
		}
//		public LogData GetLogData(string sessionID)
//		{
//			return null;
//			//			LogData logData = new LogData(this.applicationStartTime,sessionID);
//			//			//if (logDataCollection[sessionID]!=null)
//			//			//	logDataCollection.Remove(sessionID);
//			//
//			//			if (logDataCollection!=null)
//			//				logDataCollection.Add(sessionID,logData);
//			//
//			//			return logData;
//		}

		/// <summary>
		/// Writes the log data for this request to the log file
		/// </summary>
		/// <param name="fileName"></param>
		/// <param name="data"></param>
		public void WriteLog(string fileName,LogData data)
		{
			if (data == null)
				return;

			System.IO.StreamWriter logWriter = null;
			System.IO.FileStream logFile = null;

			try
			{
				if (TracingEnabled)
				{
					bool fileExist = new FileInfo(fileName).Exists;
					
					logFile = new FileStream(fileName,System.IO.FileMode.Append,System.IO.FileAccess.Write,System.IO.FileShare.ReadWrite);
					logWriter = new StreamWriter(logFile);
					if (!fileExist)
					{
						logWriter.WriteLine(LogData.GetFileHeader());
						logWriter.WriteLine(" ");						
					}
					logWriter.WriteLine(data.ToString());
					
				
					logWriter.Close();
					logFile.Close();
				}
			}
			catch (Exception exc)
			{
				Common.ApplicationLog.LogException(exc,"Handled exception: ");					
				if(logWriter != null)
					logWriter.Close();
				if(logFile != null)
					logFile.Close();
			}
		}
		#endregion

		#region Constructors/Destructors/Initializers
		/// <summary>
		/// Initializes the trace class
		/// </summary>
		private Trace()
		{
			applicationStartTime = DateTime.Now;
			if (TracingEnabled)
			{
				logDataCollection = new Hashtable();
				sessionCountCollection = new Hashtable();
			}
		}
		/// <summary>
		/// Reads from the configuration file whether the trace log is enabled
		/// </summary>
		static Trace()
		{
            traceEnabled = ParseMethods.ReadBool(System.Configuration.ConfigurationManager.AppSettings[StringConstants.CONFIG_KEY_TRACE_ENABLED]);
		}


		/// <summary>
		/// Gets the context of the current request
		/// </summary>
		/// <param name="Request"></param>
		/// <returns></returns>
		public static HttpContext GetContext(HttpRequest Request)
		{
			System.Reflection.PropertyInfo info = Request.GetType().GetProperty("Context",System.Reflection.BindingFlags.NonPublic|System.Reflection.BindingFlags.Instance);
			if (info==null)
				return null;
			HttpContext context = info.GetValue(Request,null) as HttpContext;
			return context;
		}
		#endregion

	}
	#endregion
}