using System;
using System.Data;
using System.Data.SqlClient;

using Common;

namespace DataLayer
{
	public class MSSqlHelper : IHelper
	{
		private SqlConnection sqlConnection;
		private SqlTransaction sqlTransaction;
		private SqlDataAdapter sqlDataAdapter;
		private SqlCommand sqlSelectCommand;
		private SqlCommand sqlInsertCommand;
		private SqlCommand sqlUpdateCommand;
		private SqlCommand sqlDeleteCommand;

		public IDbConnection Connection
		{
			get
			{
				return sqlConnection;
			}
		}

		public MSSqlHelper()
		{
			string connectionString = "";
			Init(connectionString);
		}

		public MSSqlHelper(string connectionString)
		{
			Init(connectionString);
		}

		public MSSqlHelper(string server, string db)
		{
			string connectionString = "Integrated Security=SSPI;Data Source=" + server + ";Initial Catalog=" + db;
			Init(connectionString);
		}

		public MSSqlHelper(string server, string db, string user, string password)
		{
			string connectionString = "data source=" + server + ";initial catalog=" + db + ";user id=" + user + ";Password=" + password;
			Init(connectionString);
		}

		public void Dispose()
		{
			if (sqlSelectCommand != null) sqlSelectCommand.Dispose();
			if (sqlInsertCommand != null) sqlInsertCommand.Dispose();
			if (sqlUpdateCommand != null) sqlUpdateCommand.Dispose();
			if (sqlDeleteCommand != null) sqlDeleteCommand.Dispose();
			if (sqlDataAdapter != null) sqlDataAdapter.Dispose();
			if (sqlTransaction != null) sqlTransaction.Dispose();
			if (sqlConnection != null) sqlConnection.Dispose();
		}

		private void Init(string connectionString)
		{
			// init connection object
			sqlConnection = new SqlConnection();
			sqlConnection.ConnectionString = connectionString;

			// init commands objects
			sqlSelectCommand = sqlConnection.CreateCommand();
			sqlInsertCommand = sqlConnection.CreateCommand();
			sqlUpdateCommand = sqlConnection.CreateCommand();
			sqlDeleteCommand = sqlConnection.CreateCommand();

			// init data adapter object
			sqlDataAdapter = new SqlDataAdapter();
			sqlDataAdapter.SelectCommand = sqlSelectCommand;
			sqlDataAdapter.InsertCommand = sqlInsertCommand;
			sqlDataAdapter.UpdateCommand = sqlUpdateCommand;
			sqlDataAdapter.DeleteCommand = sqlDeleteCommand;
		}

		public void OpenConnection()
		{
			if (sqlConnection.State == ConnectionState.Closed)
				sqlConnection.Open();
		}

		public void CloseConnection()
		{// close connection if it is possible
			try
			{
				if (sqlConnection.State != ConnectionState.Closed)
					sqlConnection.Close();
			}
			catch
			{ }//do nothing
		}

		public void BeginTransaction()
		{
			OpenConnection();
			sqlTransaction = sqlConnection.BeginTransaction();
			sqlSelectCommand.Transaction = sqlTransaction;
			sqlInsertCommand.Transaction = sqlTransaction;
			sqlUpdateCommand.Transaction = sqlTransaction;
			sqlDeleteCommand.Transaction = sqlTransaction;
		}

		public void CommitTransaction()
		{
			sqlTransaction.Commit();
		}

		public void RollbackTransaction()
		{// rollback transaction if it is possible
			try
			{
				sqlTransaction.Rollback();
			}
			catch
			{
				//do nothing
			}
		}

		public void LoadDataTableSchema(DataTable dataTable)
		{
			LoadDataTableSchema(dataTable, null);
		}

		public void LoadDataTableSchema(DataTable dataTable, string[] selectColumns)
		{
			PrepareSelectCommand(dataTable, sqlSelectCommand, selectColumns, null, null, null, null);
			sqlDataAdapter.FillSchema(dataTable, SchemaType.Source);
			CheckSchema(dataTable);
		}

		public void LoadDataTable(DataTable dataTable, string[] selectColumns, string[] filterColumns, object[] filterValues, string[] sortColumns, bool[] sortAsc, bool fillSchema)
		{
			PrepareSelectCommand(dataTable, sqlSelectCommand, selectColumns, filterColumns, filterValues, sortColumns, sortAsc);

			if (fillSchema)
				sqlDataAdapter.FillSchema(dataTable, SchemaType.Source);

			sqlDataAdapter.Fill(dataTable);

			if (fillSchema)
				CheckSchema(dataTable);
		}

		public void Update(DataTable dataTable)
		{
			PrepareUpdateCommand(dataTable, sqlUpdateCommand);
			PrepareInsertCommand(dataTable, sqlInsertCommand);
			PrepareDeleteCommand(dataTable, sqlDeleteCommand);
			sqlDataAdapter.Update(dataTable);
		}

		public void Update(DataRow[] dataRows)
		{
			if (dataRows != null && dataRows.Length > 0)
			{
				DataTable dataTable = dataRows[0].Table;
				PrepareUpdateCommand(dataTable, sqlUpdateCommand);
				PrepareInsertCommand(dataTable, sqlInsertCommand);
				PrepareDeleteCommand(dataTable, sqlDeleteCommand);

				sqlDataAdapter.Update(dataRows);
			}
		}

		public object ExecuteScalar(string sql)
		{
			bool closedConnection = (sqlConnection.State == ConnectionState.Closed);
			OpenConnection();

			sqlSelectCommand.CommandText = sql;
			object retVal = sqlSelectCommand.ExecuteScalar();

			if (closedConnection)
				CloseConnection();

			return retVal;
		}

		public int ExecuteNonQuery(string sql)
		{
			bool closedConnection = (sqlConnection.State == ConnectionState.Closed);
			OpenConnection();

			sqlSelectCommand.CommandText = sql;
			int retVal = sqlSelectCommand.ExecuteNonQuery();

			if (closedConnection)
				CloseConnection();

			return retVal;
		}

		public int Fill(DataTable dataTable, string sql)
		{
			sqlSelectCommand.CommandText = sql;
			return sqlDataAdapter.Fill(dataTable);
		}

		public int Fill(DataTable dataTable)
		{
			return sqlDataAdapter.Fill(dataTable);
		}

		public IDbCommand SelectCommand
		{ get { return sqlSelectCommand; } }

		public IDbCommand InsertCommand
		{ get { return sqlInsertCommand; } }

		public IDbCommand UpdateCommand
		{ get { return sqlUpdateCommand; } }

		public IDbCommand DeleteCommand
		{ get { return sqlDeleteCommand; } }

		public System.Data.Common.DbDataAdapter DataAdapter
		{ get { return sqlDataAdapter; } }

		public void PrepareSelectCommand(DataTable dataTable, IDbCommand sqlCommand, string[] selectColumns, string[] filterColumns, object[] filterValues, string[] sortColumns, bool[] sortAsc)
		{
			IDataParameter param;
			string sql = "select ";
			string where = "";
			string order = "";
			string sep = "";

			sqlCommand.Parameters.Clear();
			sqlCommand.CommandType = CommandType.Text;
			sqlCommand.CommandText = "";

			if (selectColumns == null)
				sql += "*";
			else
				for (int index = 0; index < selectColumns.Length; index++)
				{
					sql += sep + "[" + selectColumns[index] + "]";
					sep = ",";
				}

			sep = "";
			if (filterColumns != null)
			{
				for (int index = 0; index < filterColumns.Length; index++)
				{
					string paramName = "@Param" + index.ToString();
					where += sep + "[" + filterColumns[index] + "]=" + paramName;
					sep = " and ";

					param = sqlCommand.CreateParameter();
					param.ParameterName = paramName;
					param.SourceColumn = filterColumns[index];
					param.Value = filterValues[index];
					sqlCommand.Parameters.Add(param);
				}
			}

			sep = "";
			if (sortColumns != null)
			{
				for (int index = 0; index < sortColumns.Length; index++)
				{
					order += sep + "[" + sortColumns[index] + "] " + (sortAsc[index] ? "ASC" : "DESC");
					sep = ",";
				}
			}

			sql += " from [" + dataTable.TableName + "]";
			if (where != "") sql += " where " + where;
			if (order != "") sql += " order by " + order;

			sqlCommand.CommandText = sql;
		}

		public void PrepareChildSelectCommand(DataTable dataTable, IDbCommand sqlCommand, string[] selectColumns, string parentTableName, string parentFKName, string childFKName, string parentPKName, int parentPKValue)
		{
			string sql = "select ";
			string where = "";
			string sep = "";
			string childTableName = "[" + dataTable.TableName + "]";
			parentTableName = "[" + parentTableName + "]";
			parentFKName = "[" + parentFKName + "]";
			childFKName = "[" + childFKName + "]";
			parentPKName = "[" + parentPKName + "]";

			sqlCommand.Parameters.Clear();
			sqlCommand.CommandType = CommandType.Text;
			sqlCommand.CommandText = "";

			if (selectColumns == null)
				sql += childTableName + ".*";
			else
				for (int index = 0; index < selectColumns.Length; index++)
				{
					sql += sep + childTableName + ".[" + selectColumns[index] + "]";
					sep = ",";
				}

			where = parentTableName + "." + parentFKName + "=" + childTableName + "." + childFKName + " and ";
			where += parentTableName + "." + parentPKName + "=" + parentPKValue.ToString();

			sql += " from " + childTableName + "," + parentTableName + " where " + where;

			sqlCommand.CommandText = sql;
		}


		public void PrepareUpdateCommand(DataTable dataTable, IDbCommand sqlCommand)
		{
			IDataParameter param;
			string sql = "update [" + dataTable.TableName + "] set ";
			string sep = "";

			sqlCommand.Parameters.Clear();
			sqlCommand.CommandType = CommandType.Text;
			sqlCommand.CommandText = "";
			foreach (DataColumn col in dataTable.Columns)
			{
				if (!col.AutoIncrement)
				{
					string paramName = "@Param" + col.Ordinal.ToString();
					param = sqlCommand.CreateParameter();
					param.ParameterName = paramName;
					param.SourceColumn = col.ColumnName;
					SetSqlDbType((param as SqlParameter), col);
					sqlCommand.Parameters.Add(param);

					sql += sep + "[" + col.ColumnName + "]=" + paramName;
					sep = ",";
				}
			}

			string where = null;
			foreach (DataColumn pkColumn in dataTable.PrimaryKey)
			{
				param = sqlCommand.CreateParameter();
				string paramName = "@Original_ID" + pkColumn.Ordinal.ToString();
				string pkName = pkColumn.ColumnName;
				param.ParameterName = paramName;
				param.SourceColumn = pkName;
				param.SourceVersion = DataRowVersion.Original;
				sqlCommand.Parameters.Add(param);
				if (where != null) where += " and ";
				where += "[" + pkName + "]=" + paramName;
			}

			sql += " where " + where;
			sqlCommand.CommandText = sql;
		}

		public void PrepareInsertCommand(DataTable dataTable, IDbCommand sqlCommand)
		{
			IDataParameter param;
			string sql = "insert into [" + dataTable.TableName + "]";
			string fields = "", values = "", sep = "", autoInc = "";

			sqlCommand.Parameters.Clear();
			sqlCommand.CommandType = CommandType.Text;
			sqlCommand.CommandText = "";
			foreach (DataColumn col in dataTable.Columns)
			{
				if (!col.AutoIncrement)
				{
					string paramName = "@Param" + col.Ordinal.ToString();
					param = sqlCommand.CreateParameter();
					param.ParameterName = paramName;
					param.SourceColumn = col.ColumnName;
					SetSqlDbType((param as SqlParameter), col);
					sqlCommand.Parameters.Add(param);

					fields += sep + "[" + col.ColumnName + "]";
					values += sep + paramName;
					sep = ",";
				}
				else
					autoInc = "[" + col.ColumnName + "]";
			}

			sql += " (" + fields + ") values (" + values + ")";
			if (!(autoInc == ""))
				sql += ";select " + autoInc + sep + fields + " from [" + dataTable.TableName + "] where " + autoInc + "=SCOPE_IDENTITY()"; ;

			sqlCommand.CommandText = sql;
		}

		public void PrepareDeleteCommand(DataTable dataTable, IDbCommand sqlCommand)
		{
			sqlCommand.Parameters.Clear();
			sqlCommand.CommandType = CommandType.Text;
			sqlCommand.CommandText = "";

			string where = null;
			foreach (DataColumn pkColumn in dataTable.PrimaryKey)
			{
				IDataParameter param = sqlCommand.CreateParameter();
				string paramName = "@Original_ID" + pkColumn.Ordinal.ToString();
				string pkName = pkColumn.ColumnName;
				param.ParameterName = paramName;
				param.SourceColumn = pkName;
				param.SourceVersion = DataRowVersion.Original;
				sqlCommand.Parameters.Add(param);
				if (where != null) where += " and ";
				where += "[" + pkName + "]=" + paramName;
			}

			string sql = "delete from [" + dataTable.TableName + "] where " + where;
			sqlCommand.CommandText = sql;
		}

		private void SetSqlDbType(SqlParameter sqlParameter, DataColumn dataColumn)
		{//for more info read the topic "Using Parameters with a DataAdapter" in MSDN

			switch (dataColumn.DataType.ToString())
			{
				case "System.String":
					if (dataColumn.MaxLength > 8000)
						sqlParameter.SqlDbType = SqlDbType.NText;
					else if (dataColumn.MaxLength > 4000)
					{
						sqlParameter.SqlDbType = SqlDbType.VarChar;
						sqlParameter.Size = dataColumn.MaxLength;
					}
					else if (dataColumn.MaxLength > 0)
					{
						sqlParameter.SqlDbType = SqlDbType.NVarChar;
						sqlParameter.Size = dataColumn.MaxLength;
					}
					else
						sqlParameter.SqlDbType = SqlDbType.NVarChar;

					break;
				case "System.Byte[]":
					sqlParameter.SqlDbType = SqlDbType.Image;
					break;
				default:
					//If the parameter type is not specified, 
					//ADO.NET will infer the .NET Framework data provider type of the Parameter
					//from the .NET Framework type of the Value of the Parameter object. 
					break;
			}
		}

		public string PrepareRenameTableCommand(string oldName, string newName)
		{
			string sql = "EXECUTE sp_rename '" + oldName + "', '" + newName + "', 'object'";
			return sql;
		}

		public string PrepareRenameFieldCommand(string tableName, string oldName, string newName)
		{
			string sql = "EXECUTE sp_rename '[" + tableName + "].[" + oldName + "]', '" + newName + "', 'COLUMN'";
			return sql;
		}

		public string PrepareDropTableCommand(string tableName)
		{
			string sql = "DROP TABLE [" + tableName + "]";
			return sql;
		}

		public string PrepareDropFieldCommand(string tableName, string fieldName)
		{
			string sql = "ALTER TABLE [" + tableName + "] DROP COLUMN [" + fieldName + "]";
			return sql;
		}

		public string PrepareAlterFieldCommand(string tableName, FieldDef fieldDef)
		{
			string sql = "ALTER TABLE [" + tableName + "] ALTER COLUMN ";
			sql += PrepareFieldDefinition(fieldDef);
			return sql;
		}

		public string PrepareAddFieldCommand(string tableName, FieldDef fieldDef)
		{
			string sql = "ALTER TABLE [" + tableName + "] ADD ";
			sql += PrepareFieldDefinition(fieldDef);
			return sql;
		}

		public string PrepareFieldDefinition(FieldDef fieldDef)
		{
			string sql = "[" + fieldDef.Name + "] " + GetDBType(fieldDef.Type, fieldDef.Length);
			if (fieldDef.AutoNumber) sql += " IDENTITY (1, 1)";
			if (fieldDef.AllowNull)
				sql += " NULL";
			else
				sql += " NOT NULL";

			return sql;
		}

		public string PrepareCreateTableCommand(string tableName, FieldDef[] fields)
		{
			string sql = "CREATE TABLE [" + tableName + "] (";
			string sep1 = "", sep2 = "";
			string pk = "";

			foreach (FieldDef field in fields)
			{
				sql += sep1 + PrepareFieldDefinition(field);
				sep1 = ",";
				if (field.PrimaryKey)
				{
					pk += sep2 + "[" + field.Name + "]";
					sep2 = ",";
				}
			}

			if (pk != "")
				sql += ", CONSTRAINT [PK_" + tableName + "] PRIMARY KEY CLUSTERED (" + pk + ")";

			sql += ")";
			return sql;
		}

		private string GetDBType(DBType type, int length)
		{
			string dbType = "";
			switch (type)
			{
				case DBType.String:
					dbType = "[nvarchar] (" + length.ToString() + ")";
					break;
				case DBType.Text:
					dbType = "[ntext]";
					break;
				case DBType.Integer:
					dbType = "[int]";
					break;
				case DBType.Numeric:
					dbType = "[float]";
					break;
				case DBType.DateTime:
					dbType = "[datetime]";
					break;
				case DBType.Boolean:
					dbType = "[bit]";
					break;
			}

			return dbType;
		}

		private void CheckSchema(DataTable dataTable)
		{
			foreach (DataColumn col in dataTable.Columns)
			{
				if (col.AutoIncrement)
				{
					col.AutoIncrementSeed = -1;
					col.AutoIncrementStep = -1;
				}
			}
		}
	}
}
