using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Xml;
using System.Text;
using System.IO;

using Common;
using BusinessLayer;
using CrystalDecisions.CrystalReports.Engine;
using CrystalDecisions.Shared;

namespace FrameWork
{
    public class PageBase : Page
    {
        #region Page Constants
        public const string VIEW_STATE_F_NAME = "__ViewStateKey";
        #endregion

        #region Fields
        protected System.Collections.Specialized.NameValueCollection QueryString;
        private bool saveViewStateInSession = true;
        private static string applicationPath = null;
        private string pageFileName = null;
        private bool? isMobile = null;
        protected bool enableBrowserCaching = true; 
        #endregion

        #region Properties
        public static string ApplicationPath
        {
            get
            {
                if (applicationPath == null)
                    applicationPath = GetApplicationPath();
                return applicationPath;
            }
        }
        
        public virtual string PageFileName
        {
            get
            {
                if (pageFileName == null)
                {
                    string filePath = System.Web.HttpContext.Current.Request.FilePath.ToLower();
                    string appPath = System.Web.HttpContext.Current.Request.ApplicationPath.ToLower();

                    StringBuilder builder = new StringBuilder();
                    builder.Append(filePath);
                    builder = builder.Replace(appPath, "");
                    if (builder[0] == '.' || builder[0] == '/')
                        builder = builder.Remove(0, 1);

                    filePath = builder.ToString();
                    if (filePath.IndexOf(".aspx") < 0)
                        filePath = builder.Replace('.', '/').Append(".aspx").ToString();
                    pageFileName = filePath;
                }

                return pageFileName;
            }
        }

        protected UserData CurrentUserData
        {
            get
            {
                return BusinessLayer.UserData.CurrentUser;
            }
        }

        public bool IsMobileDevice
        {
            get
            {
                if (isMobile == null)
                {
                    try
                    {
                        System.Web.Mobile.MobileCapabilities currentCapabilities = Request.Browser as System.Web.Mobile.MobileCapabilities;
                        if (currentCapabilities == null)
                            return false;

                        return currentCapabilities.IsMobileDevice;
                    }
                    catch
                    {
                        isMobile = false;
                    }
                }

                return (bool)isMobile;
            }
        }

        public bool EnableBrowserCaching
        {
            get { return enableBrowserCaching; }
        }

        #endregion

        #region Initialization
        public PageBase()
        {

        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            if (Page.IsPostBack)
                ClearErrors();
            else
                LoadStaticData();
        }

        override protected void OnInit(EventArgs e)
        {
            CheckForUnauthorizedAccess();
            SetDataLayer();
            GetQueryString();
            SaveViewStateInSession = true;
            ClearControlStyles();

            InitPageCache();
            CurrentUserData.PageHistory.Push(this.GetType().ToString(), Request.Url.PathAndQuery);

            if (!enableBrowserCaching)
                Response.Cache.SetCacheability(HttpCacheability.NoCache);
            
            base.OnInit(e);
        }

        protected void Page_PreInit(Object sender, EventArgs e)
        {
            this.OnPreInit(sender, e);
        }

        protected virtual void CheckForUnauthorizedAccess()
        {
            if (!System.Web.HttpContext.Current.Request.IsAuthenticated)
            {
                OpenUrl("~/Login.aspx", false);
            }
        }

        protected virtual void OnPreInit(Object sender, EventArgs e)
        {
            CurrentUserData.PageHistory.Push(this.GetType().ToString(), Request.Url.PathAndQuery);
        }

        protected virtual void LoadStaticData()
        {
        
        }

        protected virtual void InitPageCache()
        {
            CurrentUserData.PageCache.Init(LanguageGroupKey);
        }

        #endregion

        #region URL & Navigation
        private static string GetApplicationPath()
        {
            string appPath = HttpContext.Current.Request.ApplicationPath + '/';
            return appPath;
        }

        protected void OpenTopUrl(string url)
        {
            if (url == null && url == "")
                url = "Default.aspx";
            Response.Clear();
            Response.Write("<script>window.top.location.href='" + ApplicationPath + url + "'</script>");
        }

        protected void OpenUrl(string url, bool encrypt)
        {
            if (encrypt)
                url = GetEncryptedUrl(url);
            Response.Redirect(url);
        }

        public string GetParameterString()
        {
            System.Text.StringBuilder builder = new System.Text.StringBuilder();
            for (int i = 0; i < QueryString.Keys.Count; i++)
            {
                if (i == 0)
                    builder.Append('?');
                else
                    builder.Append('&');

                builder.Append(QueryString.Keys[i]);
                builder.Append('=');
                builder.Append(QueryString[QueryString.Keys[i]]);
            }
            return builder.ToString();
        }

        private void GetQueryString()
        {
            if (Request.QueryString[StringConstants.QUERY_ENCRYPTED_KEY] != null)
            {
                QueryString = new System.Collections.Specialized.NameValueCollection();
                Common.Encryption decryptor = new Common.Encryption(Session.SessionID);
                string param = decryptor.Decrypt(Request.QueryString[StringConstants.QUERY_ENCRYPTED_KEY]);
                string[] arrPairs = param.Split(new char[] { '&' });
                string[] keyValue;
                foreach (string pair in arrPairs)
                {
                    keyValue = pair.Split(new char[] { '=' }, 2);
                    if (keyValue.Length != 2)
                        continue;
                    QueryString.Add(keyValue[0], keyValue[1]);
                }
            }
            else if (this is PageUnsecure)
                QueryString = new System.Collections.Specialized.NameValueCollection(Request.QueryString);
            else
                QueryString = new System.Collections.Specialized.NameValueCollection();
        }

        public static string GetEncryptedUrl(string Url)
        {
            int n = Url.IndexOf('?');
            if (n < 0)
                return Url;

            string param = Url.Substring(n + 1, Url.Length - n - 1);
            Common.Encryption encryptor = new Common.Encryption(HttpContext.Current.Session.SessionID);
            string paramEncrypted = encryptor.Encrypt(param);
            string result = Url.Substring(0, n) + "?" + StringConstants.QUERY_ENCRYPTED_KEY + "=" + paramEncrypted;
            return result;
        }

        #endregion

        protected virtual void SetDataLayer()
        {
            string conString = ApplicationConfiguration.GetConnectionString(StringConstants.CONFIG_KEY_CONNECTION_NAME);
            ConnectionParams conParams = new ConnectionParams(CurrentUserData.UserID, conString, DatabaseType.MSSQL);
            BusinessHelper.SetDataLayer(conParams);
        }

        #region Multilanguage and Control Manupulation
        public static Hashtable LoadLanguageData(string xmlFile)
        {
            Hashtable languageData = new Hashtable();

            System.Xml.XmlDocument xml = new System.Xml.XmlDocument();
            try
            {
                xml.Load(xmlFile);

                string formID;
                string itemID;
                string itemValue;

                foreach (System.Xml.XmlNode formNode in xml.LastChild.ChildNodes)
                {
                    Hashtable formData = new Hashtable();
                    formID = formNode.Attributes["Name"].Value;
                    languageData.Add(formID, formData);

                    foreach (System.Xml.XmlNode itemNode in formNode.ChildNodes)
                    {
                        itemID = itemNode.Attributes["Name"].Value;
                        itemValue = itemNode.InnerText;
                        formData.Add(itemID, itemValue);
                    }
                }
            }
            catch (Exception ex)
            {
                string msg = ex.Message;
            }	//do nothing

            return languageData;
        }

        protected override void OnPreRender(EventArgs e)
        {
            SetUIStrings();
            base.OnPreRender(e);
        }

        protected void SetUIStrings()
        {
            Hashtable formData = Multilanguage.GetGroupStrings(PageFileName);
            if (formData != null)
                SetAllControlsText(this, formData);
        }

        private void SetAllControlsText(Control control, Hashtable formData)
        {
            foreach (Control ctrl in control.Controls)
            {
                if (ctrl.ID != null && formData.Contains(ctrl.ID))
                    SetControlText(ctrl, formData[ctrl.ID].ToString());
                else
                    SetAllControlsText(ctrl, formData);
            }
        }

        
        protected virtual string GetFieldEmptyText()
        {
            return "";
        }

        public void SetControlText(object control, string text)
        {
            if (text == null || text == "")
                text = GetFieldEmptyText();
            // add more controls if necessary
            if (control is ValidationSummary)
                (control as ValidationSummary).HeaderText = text;
            else if (control is BaseValidator)
                (control as BaseValidator).ErrorMessage = text;
            else if (control is Label)
                (control as Label).Text = text;
            else if (control is Button)
                (control as Button).Text = text;
            else if (control is LinkButton)
                (control as LinkButton).Text = text;
            else if (control is RadioButton)
                (control as RadioButton).Text = text;
            else if (control is CheckBox)
                (control as CheckBox).Text = text;
            else if (control is Literal)
                (control as Literal).Text = text;
            else if (control is HyperLink)
                (control as HyperLink).Text = text;
            else if (control is HtmlInputButton)
                (control as HtmlInputButton).Value = text;
            else if (control is HtmlGenericControl)
                (control as HtmlGenericControl).InnerText = text;
        }

        public static string GetControlText(object control)
        {// add more controls if needed
            string text = "";
            if (control is ValidationSummary)
                text = (control as ValidationSummary).HeaderText;
            else if (control is BaseValidator)
                text = (control as BaseValidator).ErrorMessage;
            else if (control is Label)
                text = (control as Label).Text;
            else if (control is Button)
                text = (control as Button).Text;
            else if (control is LinkButton)
                text = (control as LinkButton).Text;
            else if (control is RadioButton)
                text = (control as RadioButton).Text;
            else if (control is CheckBox)
                text = (control as CheckBox).Text;
            else if (control is Literal)
                text = (control as Literal).Text;
            else if (control is HyperLink)
                text = (control as HyperLink).Text;
            else if (control is HtmlInputButton)
                text = (control as HtmlInputButton).Value;

            return text;
        }

        protected void SetControlColor(object control, Color color)
        {
            if (control is WebControl)
                (control as WebControl).ForeColor = color;
            else if (control is HtmlControl)
                (control as HtmlControl).Attributes[""] = color.ToString();
        }

        protected virtual void ShowSingleError(Label lbl, WebControl txt, string errMsg)
        {
            CustomValidator validator = new CustomValidator();
            SetControlStyle(lbl);
            SetControlStyle(txt);

            validator.ErrorMessage = "<B>" + GetControlText(lbl) + "</B> " + GetTranslatedError(errMsg);
            validator.IsValid = false;
            validator.Visible = false;

            this.Controls.Add(validator);
        }

        protected virtual void ShowSingleError(Label lbl, WebControl txt, string errMsg, Label errDisplay)
        {
            SetControlStyle(lbl);
            SetControlStyle(txt);
            if (errDisplay.Text != "")
                errDisplay.Text += "<BR>";
            errDisplay.Text += "<B>" + GetControlText(lbl) + "</B> " + GetTranslatedError(errMsg);
        }

        protected void SetControlStyle(object control)
        {
            if (!(control == null))
                if (control is Label)
                    ((Label)control).ForeColor = Color.FromName("#ED1B23");

                else if ((control is TextBox) || (control is DropDownList) || (control is CheckBox))
                    ((WebControl)control).BackColor = Color.FromName("#FFFFC0");
        }

        protected string LanguageGroupKey
        {
            get
            {
                return GetLanguageGroupKey(this.GetType().BaseType);
            }
        }

        protected string GetLanguageGroupKey(Type type)
        {
            return type.Namespace + '.' + type.Name;
        }

        protected string GetTranslatedMessage(string err)
        {
            string translatedMsg;
            int n = err.IndexOf(":");

            if (n >= 0)
            {
                string key = err.Substring(0, n);
                translatedMsg = Multilanguage.GetTranslatedString("Messages", key);

                if (translatedMsg == StringConstants.NOT_TRANSLATED)
                    translatedMsg = err.Substring(n + 1); // return original message
            }
            else
                translatedMsg = err; // return original message

            return translatedMsg;
        }

        public static string GetTranslatedError(string err)
        {
            string translatedMsg;
            int n = err.IndexOf(":");

            if (n >= 0)
            {
                string key = err.Substring(0, n);
                translatedMsg = Multilanguage.GetTranslatedString(StringConstants.MESSAGES, key);

                if (translatedMsg == StringConstants.NOT_TRANSLATED)
                    translatedMsg = err.Substring(n + 1); // return original message
            }
            else
                translatedMsg = err; // return original message

            return translatedMsg;
        }
        public void ShowGeneralError(string errMsg)
        {
            CustomValidator validator = new CustomValidator();
            validator.ErrorMessage = GetTranslatedError(errMsg);
            validator.IsValid = false;
            validator.Visible = false;

            this.Controls.Add(validator);
        }

        public void ShowGeneralError(string errMsg, Label errDisplay)
        {
            errDisplay.Text = GetTranslatedError(errMsg);
        }

        #endregion

        #region VIEW STATE MANAGEMENT
        protected override object LoadPageStateFromPersistenceMedium()
        {
            if (saveViewStateInSession)
                return LoadViewState();
            else
                return base.LoadPageStateFromPersistenceMedium();
        }

        protected override void SavePageStateToPersistenceMedium(object viewState)
        {
            if (saveViewStateInSession)
                SaveViewState(viewState);
            else
                base.SavePageStateToPersistenceMedium(viewState);
        }

        private object LoadViewState()
        {
            if (Request.Form[VIEW_STATE_F_NAME] == null) return null;

            LosFormatter formatter = new LosFormatter();

            long requestNumber = ParseMethods.ReadLong(Request.Form[VIEW_STATE_F_NAME].ToString());
            string viewState = ViewStateMgr.GetViewStateMgr().GetViewState(requestNumber);

            if (viewState == null)
                return null;
            else
                return formatter.Deserialize(viewState);
        }

        private void SaveViewState(object viewState)
        {
            LosFormatter formatter = new LosFormatter();

            StringBuilder builder = new StringBuilder();
            StringWriter writer = new StringWriter(builder);
            formatter.Serialize(writer, viewState);
            long requestNumber = ViewStateMgr.GetViewStateMgr().SaveViewState(builder.ToString());

            ClientScript.RegisterHiddenField(VIEW_STATE_F_NAME, requestNumber.ToString());
        }

        /// <summary>
        /// Indicates wheter to save page's ViewState to Session or not.
        /// On web pages that will be used on mobile devices it should be true.
        /// </summary>
        /// <remarks>Prefer setting this property through desinger rather than in code to ensure it is set on time.</remarks>
        [Browsable(true)]
        [Description(@"Indicates wheter to save page's ViewState to Session or not.
	On web pages that will be used on mobile devices it should be true.")]
        [DefaultValue(false)]
        public bool SaveViewStateInSession
        {
            set { saveViewStateInSession = value; }
            get { return saveViewStateInSession; }
        }
        #endregion

        #region UI Manipulation
        protected virtual void ClearControlStyles()
        { 
            
        }

        protected virtual void ClearControlStyle(object ctrl)
        {
            if (ctrl is WebControl)
                ((WebControl)ctrl).ForeColor = Color.Black;
        }

        protected virtual void ClearErrors()
        { }
        #endregion

        #region Export
        public void ExportToExcel(DataGrid dgData)
        {
            string data = ExportHTML(dgData);
            if (data == "") return;

            string fileName = DateTime.Today.Year.ToString() + DateTime.Today.Month.ToString("00") + DateTime.Today.Day.ToString("00") + ".xls";

            StringBuilder builder = new StringBuilder();
            builder.Append(@"<!DOCTYPE HTML PUBLIC ""-//W3C//DTD HTML 4.0 Transitional//EN/""><html><head><title>");
            builder.Append(fileName);
            builder.Append(@"</title><meta HTTP-EQUIV=""Content-Type"" CONTENT=""text/html; charset=UTF-8""></head><body>");
            builder.Append(data);
            builder.Append("</body></html>");

            Response.Clear();
            Response.ContentType = "application/vnd.ms-excel";
            Response.AddHeader("Content-Disposition", "Filename=" + fileName);
            Response.Charset = "";
            Response.Write(builder.ToString());
            Response.End();
        }

        public string ExportHTML(DataGrid data)
        {
            if (data == null || data.Items.Count == 0) return "";

            StringBuilder builder = new StringBuilder();
            StringBuilder cellTextBuilder = new StringBuilder();

            builder.Append("<HTML><BODY><TABLE border=\"1\" borderColor=\"black\">");

            Control gridRow, gridCell;

            for (int i = 0; i < data.Controls[0].Controls.Count; i++)
            {
                builder.Append("<TR>");
                gridRow = data.Controls[0].Controls[i];
                for (int j = 0; j < gridRow.Controls.Count; j++)
                {
                    if (data.Columns[j].HeaderText == "")
                        continue;

                    gridCell = gridRow.Controls[j];

                    cellTextBuilder.Length = 0;
                    foreach (Control ctrl in gridCell.Controls)
                    {
                        if (ctrl is LiteralControl || ctrl is LinkButton)
                            continue;
                        cellTextBuilder.Append(GetControlText(ctrl));
                        cellTextBuilder.Append(' ');
                    }

                    builder.Append("<TD>");
                    builder.Append(cellTextBuilder.ToString());
                    builder.Append("</TD>");
                }

                builder.Append("</TR>");
            }

            builder.Append("<TABLE></BODY></HTML>");

            return builder.ToString();
        }

        public virtual void ExportToPDF(ReportClass report, string name)
        {
            MemoryStream stream = (MemoryStream)report.ExportToStream(ExportFormatType.PortableDocFormat);
            Response.Clear();
            Response.Charset = "";
            Response.Buffer = true;
            Response.ContentType = "application/pdf";
            Response.AddHeader("content-disposition", "attachment;filename=" + name + "_" + FormatMethods.GetDBStringDate(DateTime.Today) + ".pdf");
            Response.BinaryWrite(stream.ToArray());
            Response.End();
        }
        #endregion
    }
}

